# -*- coding: utf-8 -*-

# Unimail 0.11 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, configuration

def get_current_user_homedir():
    """
    Devuelve la ruta del directorio del usuario que ha ejecutado el programa.
    """

    homeDir = os.path.expanduser("~")
    return homeDir

def get_data_dir():
    """
    Devuelve la ruta del directorio de datos de Unimail.

    Debe haber uno por cada usuario distinto que ejecuta Unimail.
    """

    homeDir = get_current_user_homedir()
    dataDir = os.path.join(homeDir, ".unimail")
    return dataDir

def get_config_dir():
    """
    Devuelve la ruta del directorio de configuración.

    Está dentro del directorio de datos.
    """

    dataDir = get_data_dir()
    configDir = os.path.join(dataDir, "config")
    return configDir

def get_accountdata_file():
    """
    Devuelve la ruta del archivo donde están guardados los datos de la
    cuenta de e-mail del usuario (donde está serializado el objeto de tipo
    `configuration.AccountData`).

    Está dentro del directorio de configuración.
    """

    configDir = get_config_dir()
    accountDataDir = os.path.join(configDir, "account-data")
    return accountDataDir

def get_temp_dir():
    """
    Devuelve la ruta del directorio temporal.
    
    En este directorio se guardan temporalmente las partes de los
    archivos a descargar o enviar.

    Está dentro del directorio de datos.
    """

    dataDir = get_data_dir()
    tempDir = os.path.join(dataDir, "temp")
    return tempDir

def get_default_inbox_dir():
    """
    Devuelve la ruta del directorio de bandeja de entrada.

    En este directorio se almacenarán los archivos recibidos completamente."
    """

    homeDir = get_current_user_homedir()
    inboxDir = os.path.join(homeDir, "unimail-inbox")
    return inboxDir

def check_dirs():
    """
    Comprueba que todos los directorios de datos usados por Unimail existen y,
    en caso de que no existan, los crea.
    """

    dataDir = get_data_dir()
    if not (os.path.isdir(dataDir) and not os.path.islink(dataDir)):
        os.mkdir(dataDir)
    configDir = get_config_dir()
    if not (os.path.isdir(configDir) and not os.path.islink(configDir)):
        os.mkdir(configDir)
    accountDataFile = get_accountdata_file()
    configManag = configuration.ConfigurationManager()
    if not (os.path.isfile(accountDataFile) and not os.path.islink(accountDataFile)):
        defaultAccData = configManag.get_default_account_data()
        configManag.save_account_data(defaultAccData)
    tempDir = get_temp_dir()
    if not (os.path.isdir(tempDir) and not os.path.islink(tempDir)):
        os.mkdir(tempDir)
    accountData = configManag.load_account_data()
    inboxDir = accountData.inboxDir
    if not (os.path.isdir(inboxDir) and not os.path.islink(inboxDir)):
        os.mkdir(inboxDir)
