#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.11 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, pygtk, gtk, gtk.glade, sysutils, guiutils, configuration, fileslogic, imap, pop, smtp, imaplogic, poplogic
import smtplogic, syspaths
import gettext

class WinMain:
    """
    Clase que representa la ventana principal de la interfaz de usuario gráfica.
    """

    def __init__(self):
        self.gladeFile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladeFile)
        self.window = self.wTree.get_widget("WinMain")
        dic = { "on_mitemexit_activate" : self.mitemexit_activate,
        "on_mitemaccount_activate" : self.mitemaccount_activate,
        "on_mitemabout_activate" : self.mitemabout_activate,
        "on_butrefresh_clicked" : self.butrefresh_clicked,
        "on_butdownload_clicked" : self.butdownload_clicked,
        "on_trvinbox_cursor_changed" : self.trvinbox_cursor_changed,
        "on_butbrowse_clicked" : self.butbrowse_clicked,
        "on_entfile_changed" : self.entfile_changed,
        "on_entsendto_changed" : self.entsendto_changed,
        "on_butsend_clicked" : self.butsend_clicked,
        "on_winmain_destroy" :  self.winmain_destroy} # gtk.main_quit
        self.wTree.signal_autoconnect(dic)
        
        self.labSessionState = self.wTree.get_widget("labSessionState")
        self.trvInbox = self.wTree.get_widget("trvInbox")
        
        column1 = gtk.TreeViewColumn("ID", gtk.CellRendererText(), text=0)
        column1.set_resizable(True)
        self.trvInbox.append_column(column1)
        
        column2 = gtk.TreeViewColumn("File", gtk.CellRendererText(), text=1)
        column2.set_resizable(True)
        self.trvInbox.append_column(column2)
        
        column3 = gtk.TreeViewColumn("From", gtk.CellRendererText(), text=2)
        column3.set_resizable(True)
        self.trvInbox.append_column(column3)
        
        self.trvInboxModel = gtk.ListStore(int, str, str)
        self.trvInbox.set_model(self.trvInboxModel)
        
        self.progbProgress = self.wTree.get_widget("progbProgress")
        self.labProgress = self.wTree.get_widget("labProgress")
        self.statbMain = self.wTree.get_widget("statbMain")
        self.pan3 = self.wTree.get_widget("pan3")
        self.butRefresh = self.wTree.get_widget("butRefresh")
        self.butDownload = self.wTree.get_widget("butDownload")
        self.mitemConfiguration = self.wTree.get_widget("mitemConfiguration")
        self.entFile = self.wTree.get_widget("entFile")
        self.entSendTo = self.wTree.get_widget("entSendTo")
        self.butSend = self.wTree.get_widget("butSend")

        confManag = configuration.ConfigurationManager()
        self.accountData = confManag.load_account_data()

        self.update_interface()
        self.update_logic()

    def winmain_destroy(self, widget):
        exit()

    def update_interface(self):
        if self.accountData.address == "":
            self.labSessionState.set_text(_("Set your account configuration first"))
        else:
            self.labSessionState.set_text(self.accountData.address)

        cond1 = (self.accountData.address != "")
        cond2 = (self.accountData.inboxSystem == "imap" and self.accountData.imapServer != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popServer != "")
        cond3 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPort != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPort != "")
        cond4 = (self.accountData.inboxSystem == "imap" and self.accountData.imapUser != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popUser != "")
        cond5 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPassword != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPassword != "")

        if cond1 and cond2 and cond3 and cond4 and cond5:
            self.labSessionState.set_text(self.accountData.address)
            self.butRefresh.set_sensitive(True)
        else:
            self.butRefresh.set_sensitive(False)

    def update_logic(self):
        self.filesLogic = fileslogic.FilesLogic(self.accountData)
        
        if self.accountData.inboxSystem == "imap":
            self.emailInboxAccount = imap.ImapAccount(self.accountData.imapServer, self.accountData.imapSsl,
                self.accountData.imapPort, self.accountData.imapUser, self.accountData.imapPassword)
            self.emailInboxLogic = imaplogic.ImapLogic(self.emailInboxAccount)
        else:
            self.emailInboxAccount = pop.PopAccount(self.accountData.popServer, self.accountData.popSsl,
                self.accountData.popPort, self.accountData.popUser, self.accountData.popPassword)
            self.emailInboxLogic = poplogic.PopLogic(self.emailInboxAccount)

    def entfile_changed(self, widget):
        self.check_send_data()
    
    def entsendto_changed(self, widget):
        self.check_send_data()

    def check_send_data(self):
        cond1 = self.entFile.get_text() != ""
        cond2 = self.entSendTo.get_text() != ""
        self.butSend.set_sensitive(cond1 and cond2)

    def mitemexit_activate(self, widget):
        exit()
        
    def mitemaccount_activate(self, widget):
        dialAccount = DialAccount(self.accountData)
        result, data = dialAccount.run()
        if result == 0:
            self.accountData = data
            confManag = configuration.ConfigurationManager()
            confManag.save_account_data(self.accountData)
            self.update_interface()
            self.update_logic()
        
    def mitemabout_activate(self, widget):
        dialAbout = DialAbout()
        dialAbout.run()
    
    def trvinbox_cursor_changed(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        cond = len(selection) > 0
        self.butDownload.set_sensitive(cond)

    def butrefresh_clicked(self, widget):
        self.refresh_inbox()        

    def butdownload_clicked(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        idSelected = selection.get_value(iterator, 0)
        self.download_file(idSelected)

    def refresh_inbox(self):        
        (guiutils.GeneratorTask(self.refresh_inbox_count, self.set_progress_value, self.refresh_inbox_finish)).start()

    def refresh_inbox_count(self):
        try:
            self.trvInboxModel.clear()
            self.pan3.set_sensitive(False)
            self.mitemConfiguration.set_sensitive(False)
            self.labProgress.set_text(_("Connecting..."))
            self.emailInboxAccount.connect()
            self.labProgress.set_text(_("Searching..."))
            self.butDownload.set_sensitive(False)
            self.progbProgress.show()

            for progress in self.emailInboxLogic.refresh():
                yield progress

            self.inboxFileList = self.emailInboxLogic.get_unimail_files_list()

            for i in range(0, len(self.inboxFileList)):
                self.trvInboxModel.append([i+1, self.inboxFileList[i]["file"], self.inboxFileList[i]["from"]])
        except:
            raise Exception(_("Error refreshing inbox"))

    def refresh_inbox_finish(self):
        self.progbProgress.hide()
        self.labProgress.set_text("")
        self.pan3.set_sensitive(True)
        self.mitemConfiguration.set_sensitive(True)
        self.labProgress.set_text("")

    def download_file(self, idSelected):
        (guiutils.GeneratorTask(self.download_file_count, self.set_progress_value, self.download_file_finish)).start(idSelected)

    def download_file_count(self, idSelected):
        self.mitemConfiguration.set_sensitive(False)
        self.pan3.set_sensitive(False)
        self.labProgress.set_text(_("Downloading..."))

        self.filesLogic.remove_temp_files()
        self.progbProgress.show()

        for progress in self.emailInboxLogic.download_file(self.inboxFileList[idSelected - 1]["index"]):
            yield progress

    def set_progress_value(self, value):
        self.progbProgress.set_fraction(float(value)/100)

    def download_file_finish(self):
        self.filesLogic.join_temp_files()
        self.filesLogic.remove_temp_files()
        self.labProgress.set_text(_("Download completed"))
        self.progbProgress.hide()
        self.mitemConfiguration.set_sensitive(True)
        self.pan3.set_sensitive(True)

    def butsend_clicked(self, widget):
        filePath = self.entFile.get_text()
        fromAddress = self.accountData.address
        toAddress = self.entSendTo.get_text()
        self.send_file(filePath, fromAddress, toAddress)

    def send_file(self, filePath, fromAddress, toAddress):
        (guiutils.GeneratorTask(self.send_file_count, self.set_progress_value, self.send_file_finish)).start(filePath, fromAddress, toAddress)

    def send_file_count(self, filePath, fromAddress, toAddress):
        self.mitemConfiguration.set_sensitive(False)
        self.pan3.set_sensitive(False)
        self.labProgress.set_text(_("Connecting..."))
        emailOutAccount = smtp.SmtpAccount(self.accountData.smtpServer, self.accountData.smtpTls, self.accountData.smtpPort,
            self.accountData.smtpUser, self.accountData.smtpPassword)
        emailOutAccount.connect()
        emailOutLogic = smtplogic.SmtpLogic(emailOutAccount, self.accountData.partWaitTime)

        self.filesLogic.remove_temp_files()
        self.filesLogic.split_file(filePath)
        self.progbProgress.show()
        self.labProgress.set_text(_("Sending..."))

        for progress in emailOutLogic.send_file(filePath, fromAddress, toAddress):
            yield progress

        emailOutAccount.close()

    def send_file_finish(self):
        self.filesLogic.remove_temp_files()
        self.labProgress.set_text(_("Send completed"))
        self.progbProgress.hide()
        self.mitemConfiguration.set_sensitive(True)
        self.pan3.set_sensitive(True)
        self.entFile.set_text("")
        self.entSendTo.set_text("")

    def butbrowse_clicked(self, widget):
        dialsend_file = DialSendFile()
        result, fileName = dialsend_file.run()
        if result == 0:
            self.entFile.set_text(fileName)

class DialAccount:
    """
    Clase que representa la ventana de configuración de la cuenta de e-mail de la interfaz de usuario gráfica.
    """

    def __init__(self, accountData):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladefile, "DialAccount")
        self.window = self.wTree.get_widget("DialAccount")
        dic = { "on_butinboxfolderpath_clicked" : self.butinboxfolderpath_clicked }
        self.wTree.signal_autoconnect(dic)

        self.entAddress = self.wTree.get_widget("entAddress")
        self.entAddress.set_text(accountData.address)
        self.radImap = self.wTree.get_widget("radImap")
        self.radPop = self.wTree.get_widget("radPop")

        if (accountData.inboxSystem == "imap"):
            self.radImap.set_active(True)
        else:
            self.radImap.set_active(False)

        self.entImapServer = self.wTree.get_widget("entImapServer")
        self.entImapServer.set_text(accountData.imapServer)
        self.chkImapSsl = self.wTree.get_widget("chkImapSsl")
        self.chkImapSsl.set_active(accountData.imapSsl)
        self.entImapPort = self.wTree.get_widget("entImapPort")
        self.entImapPort.set_text(str(accountData.imapPort))
        self.entImapUser = self.wTree.get_widget("entImapUser")
        self.entImapUser.set_text(accountData.imapUser)
        self.entImapPass = self.wTree.get_widget("entImapPass")
        self.entImapPass.set_text(accountData.imapPassword)
        self.entImapPass.set_visibility(False)
        
        self.entPopServer = self.wTree.get_widget("entPopServer")
        self.entPopServer.set_text(accountData.popServer)
        self.chkPopSsl = self.wTree.get_widget("chkPopSsl")
        self.chkPopSsl.set_active(accountData.popSsl)
        self.entPopPort = self.wTree.get_widget("entPopPort")
        self.entPopPort.set_text(str(accountData.popPort))
        self.entPopUser = self.wTree.get_widget("entPopUser")
        self.entPopUser.set_text(accountData.popUser)
        self.entPopPass = self.wTree.get_widget("entPopPass")
        self.entPopPass.set_text(accountData.popPassword)
        self.entPopPass.set_visibility(False)

        self.spinbPartSize = self.wTree.get_widget("spinbPartSize")
        self.spinbPartSize.set_value(accountData.partSize / (1024 * 1024))
        self.spinbPartWaitTime = self.wTree.get_widget("spinbPartWaitTime")
        self.spinbPartWaitTime.set_value(accountData.partWaitTime)

        self.entInboxFolderPath = self.wTree.get_widget("entInboxFolderPath")
        self.entInboxFolderPath.set_text(accountData.inboxDir)

        self.entSmtpServer = self.wTree.get_widget("entSmtpServer")
        self.entSmtpServer.set_text(accountData.smtpServer)
        self.chkSmtpTls = self.wTree.get_widget("chkSmtpTls")
        self.chkSmtpTls.set_active(accountData.smtpTls)
        self.entSmtpPort = self.wTree.get_widget("entSmtpPort")
        self.entSmtpPort.set_text(str(accountData.smtpPort))
        self.entSmtpUser = self.wTree.get_widget("entSmtpUser")
        self.entSmtpUser.set_text(accountData.smtpUser)
        self.entSmtpPass = self.wTree.get_widget("entSmtpPass")
        self.entSmtpPass.set_text(accountData.smtpPassword)
        self.entSmtpPass.set_visibility(False)

        self.accountData = None

    def run(self):
        self.result = self.window.run()
        
        address = self.entAddress.get_text()
        
        if self.radImap.state == 1:
            inboxSystem = "imap"
        else:
            inboxSystem = "pop"

        imapServer = self.entImapServer.get_text()
        imapSsl = self.chkImapSsl.get_active()
        imapPort = int(self.entImapPort.get_text())
        imapUser = self.entImapUser.get_text()
        imapPassword = self.entImapPass.get_text()

        popServer = self.entPopServer.get_text()
        popSsl = self.chkPopSsl.get_active()
        popPort = int(self.entPopPort.get_text())
        popUser = self.entPopUser.get_text()
        popPassword = self.entPopPass.get_text()

        smtpServer = self.entSmtpServer.get_text()
        smtpTls = self.chkSmtpTls.get_active()
        smtpPort = int(self.entSmtpPort.get_text())
        smtpUser = self.entSmtpUser.get_text()
        smtpPassword = self.entSmtpPass.get_text()

        partSize = int(self.spinbPartSize.get_value() * (1024*1024))
        partWaitTime = int(self.spinbPartWaitTime.get_value())

        inboxDir = self.entInboxFolderPath.get_text()

        self.accountData = configuration.AccountData(address, inboxSystem,
        imapServer, imapSsl, imapPort, imapUser, imapPassword,
        popServer, popSsl, popPort, popUser, popPassword,
        smtpServer, smtpTls, smtpPort, smtpUser, smtpPassword,
        partSize, partWaitTime,
        inboxDir)
        
        self.window.destroy()
        return self.result, self.accountData

    def butinboxfolderpath_clicked(self, widget):
        dialInboxFolder = DialInboxFolder(self.entInboxFolderPath.get_text())
        result, dirpath = dialInboxFolder.run()
        if result == 0:
            self.entInboxFolderPath.set_text(dirpath)

class DialAbout:
    """
    Clase que representa la ventana de información sobre Unimail de la interfaz de usuario gráfica.
    """

    def __init__(self):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladefile, "DialAbout")
        self.window = self.wTree.get_widget("DialAbout")
    
    def run(self):
        self.result = self.window.run()
        self.window.destroy()
        return self.result

class DialSendFile:
    """
    Clase que representa la ventana de selección del archivo a enviar de la interfaz de usuario gráfica.
    """

    def __init__(self):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladefile, "DialSendFile")
        self.window = self.wTree.get_widget("DialSendFile")
    
    def run(self):
        self.result = self.window.run()
        self.fileName = self.window.get_filename()
        self.window.destroy()
        return self.result, self.fileName

class DialError:
    """
    Clase que representa la ventana de los mensajes de error de la interfaz de usuario gráfica.
    """

    def __init__(self, message):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladefile, "DialError")
        self.window = self.wTree.get_widget("DialError")
        
        self.labMsg = self.wTree.get_widget("labMsg")
        self.labMsg.set_text(message)

    def run(self):
        self.window.run()
        self.window.destroy()
        return self.result

class DialInboxFolder:
    """
    Clase que representa la ventana de selección de la carpeta de bandeja de entrada de la interfaz de usuario gráfica.
    """

    def __init__(self, currentInboxDir):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "presentation.glade")
        self.wTree = gtk.glade.XML(self.gladefile, "DialInboxFolder")
        self.window = self.wTree.get_widget("DialInboxFolder")
        self.window.set_current_folder(currentInboxDir)

    def run(self):
        self.result = self.window.run()
        self.dirpath = self.window.get_current_folder()
        self.window.destroy()
        return self.result, self.dirpath

if __name__ == "__main__":
    _ = gettext.gettext
    gettext.textdomain("unimail")
    gettext.bindtextdomain("unimail", "./languages")
    gtk.glade.textdomain("unimail")
    gtk.glade.bindtextdomain("unimail", "./languages")

    syspaths.check_dirs()

    window = WinMain()
    gtk.main()
